@@ -5,11 +5,9 @@ and the methods to authenticate, get metrics, and get data from the 24SEA API.
 from __future__ import annotations
 
 import datetime
-import os
-from typing import Any, Dict, List, Optional, Union
+from typing import Dict, List, Optional, Union
 from warnings import simplefilter
 
-import httpx
 import pandas as pd
 
 # Local imports
@@ -62,28 +60,36 @@ class DataSignals:
             API client instance for making requests.
         """
 
-        self._obj = pandasdata
-        self.base_url: str = f"{U.BASE_URL}datasignals/"
-        self._username: Optional[str] = None
-        self._password: Optional[str] = None
-        self._auth: Optional[httpx.BasicAuth] = None
-        self._authenticated: bool = False
-        self._selected_metrics: Optional[pd.DataFrame] = None
-        self.__api = C.API()
+        self._obj: pd.DataFrame = pandasdata
+        self.__api: C.API = C.API()
+        self.__sync_with_api()
 
-    def _lazy_authenticate(self) -> Any:
+    def __sync_with_api(self):
+        """Sync the local DataFrame with the API."""
+        self.base_url = self.__api.base_url
+        self._username = self.__api._username
+        self._password = self.__api._password
+        self._auth = self.__api._auth
+        self._authenticated = self.__api._authenticated
+        self._metrics_overview = self.__api._metrics_overview
+        self._selected_metrics = self.__api._selected_metrics
+
+    def _lazy_authenticate(self) -> bool:
         """Attempt authentication using environment variables"""
-        username = os.getenv("24SEA_API_USERNAME")
-        password = os.getenv("24SEA_API_PASSWORD")
-        if username and password:
-            return self.authenticate(username, password)
-        return
+        self._authenticated = self.__api._lazy_authenticate()
+        return self._authenticated
 
     @property
     def authenticated(self) -> bool:
         """Return the authentication status."""
         return self._authenticated
 
+    @property
+    @U.require_auth
+    def metrics_overview(self) -> Optional[pd.DataFrame]:
+        """Get the metrics overview DataFrame."""
+        return self.__api.metrics_overview
+
     @U.validate_call(config=dict(arbitrary_types_allowed=True))
     def authenticate(
         self,
@@ -106,38 +112,22 @@ class DataSignals:
         pd.DataFrame
             The authenticated DataFrame.
         """
-        self._username = username
-        self._password = password
-        self._auth = httpx.BasicAuth(username, password)
-        if self.__api._authenticated:
-            self._authenticated = True
-            return self._obj
-        self.__api.authenticate(username, password)
-        self._authenticated = True
+        if not self._lazy_authenticate():
+            self.__api = self.__api.authenticate(
+                username, password, __metrics_overview
+            )
+        self.__sync_with_api()
         return self._obj
 
     @property
+    @U.require_auth
     def selected_metrics(self) -> pd.DataFrame:
         """Return the selected metrics for the query."""
         # Get the selected metrics as the self._obj columns that are available
         # in the metrics_overview DataFrame
-        if self._selected_metrics is not None:
-            # Check that the index is set to the metric column
-            if "metric" not in self._selected_metrics.index:
-                if "metric" not in self._selected_metrics.columns:
-                    raise E.DataSignalsError(
-                        "\033[31mThe \033[1mselected_metrics\033[22m DataFrame "
-                        "must have the \033[1mmetric\033[22m column as the index."
-                    )
-                self._selected_metrics.set_index("metric", inplace=True)
-            return self._selected_metrics
         return self.__api.selected_metrics(self._obj)
 
-    @selected_metrics.setter
-    def selected_metrics(self, value: pd.DataFrame) -> None:
-        """Set the selected metrics for the query."""
-        self._selected_metrics = value
-
+    @U.require_auth
     @U.validate_call
     def __get_data(
         self,
@@ -292,33 +282,13 @@ class DataSignals:
             if l_ not in __dict[s_]:
                 __dict[s_][l_] = _df
             # Pass also the authentication, and __api to the DataFrame
-            if (
-                os.getenv("24SEA_API_USERNAME") is None
-                and os.getenv("24SEA_API_PASSWORD") is None
-            ):
-                try:
-                    __dict[s_][l_].datasignals.__api = C.API()
-                    __dict[s_][l_].datasignals.__api._username = self._username
-                    __dict[s_][l_].datasignals.__api._password = self._password
-                    __dict[s_][l_].datasignals.__api._auth = self._auth
-                    __dict[s_][
-                        l_
-                    ].datasignals.__api._authenticated = self._authenticated
-                    __dict[s_][l_].datasignals._username = self._username
-                    __dict[s_][l_].datasignals._password = self._password
-                    __dict[s_][l_].datasignals._auth = self._auth
-                    __dict[s_][
-                        l_
-                    ].datasignals._authenticated = self._authenticated
-
-                    __dict[s_][
-                        l_
-                    ].datasignals.__api.metrics_overview = (
-                        self.__api.metrics_overview
-                    )
-                except AttributeError as ae:
-                    if "datasignals" in str(ae):
-                        pass
+            ds = getattr(__dict[s_][l_], "datasignals", None)
+            if not ds:
+                __dict[s_][l_].datasignals = DataSignals(__dict[s_][l_])
+            ds = __dict[s_][l_].datasignals
+            # pass down the same API client and sync its state
+            ds.__api = self.__api
+            ds.__sync_with_api()
         return __dict
 
     def get_data(
